package com.ruoyi.system.mapper;


import com.ruoyi.common.core.domain.entity.TbSysArea;
import com.ruoyi.common.core.domain.entity.TbSysAreaUser;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 区域Mapper接口
 *
 * @author ruoyi
 * @date 2023-08-01
 */
public interface TbSysAreaMapper
{
    /**
     * 查询区域
     *
     * @param id 区域主键
     * @return 区域
     */
    public TbSysArea selectTbSysAreaById(Long id);

    /**
     * 查询区域列表
     *
     * @param tbSysArea 区域
     * @return 区域集合
     */
    public List<TbSysArea> selectTbSysAreaList(TbSysArea tbSysArea);

    /**
     * 新增区域
     *
     * @param tbSysArea 区域
     * @return 结果
     */
    public int insertTbSysArea(TbSysArea tbSysArea);

    /**
     * 修改区域
     *
     * @param tbSysArea 区域
     * @return 结果
     */
    public int updateTbSysArea(TbSysArea tbSysArea);

    /**
     * 删除区域
     *
     * @param id 区域主键
     * @return 结果
     */
    public int deleteTbSysAreaById(Long id);

    /**
     * 批量删除区域
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTbSysAreaByIds(Long[] ids);
    /**
     * 区域ID查询区域负责人昵称
     * @param areaIdList 区域ID
     * @return
     */
    List<TbSysAreaUser> selectUserNickNameByAreaIdList(@Param("areaIdList") List<Long> areaIdList);
    /**
     * 统计生产区域数量：企业下
     *
     * @param companyCode 企业编码
     * @return
     */
    Integer selectAreaCount(@Param("companyCode") String companyCode);
}
